(**************************************************************************)
(*  The CDuce compiler                                                    *)
(*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
(**************************************************************************)

(** Components of XML Schema
  Reference: http://www.w3.org/TR/xmlschema-1/
*)
open Ns
open Encodings

type xs_nonNegativeInteger = Big_int.big_int
type white_space_handling = [ `Preserve | `Replace | `Collapse ]
type facets = {
  length: (xs_nonNegativeInteger * bool) option;      (* length, fixed *)
  minLength: (xs_nonNegativeInteger * bool) option;   (* length, fixed *)
  maxLength: (xs_nonNegativeInteger * bool) option;   (* length, fixed *)
  enumeration: Value.ValueSet.t option;
  whiteSpace: white_space_handling * bool;           (* handling, fixed *)
  maxInclusive: (Value.t * bool) option;        (* max, fixed *)
  maxExclusive: (Value.t * bool) option;        (* max, fixed *)
  minInclusive: (Value.t * bool) option;        (* min, fixed *)
  minExclusive: (Value.t * bool) option;        (* min, fixed *)
}

type value_constraint =
  | No_constraint
  | Default of Utf8.t
  | Fixed of Utf8.t

type 'a ptr = 'a option ref

type attribute_declaration = {
  ad_name: qname;
  ad_type: simple_type_definition ptr;
  ad_cstr: value_constraint;
}
and element_declaration = {
  ed_name: qname;
  ed_type: type_definition;
  ed_cstr: value_constraint;
  ed_nillable: bool;
}
and complex_type_definition = {
  xt_name: qname;
  xt_base: type_definition;
  xt_derivation: derivation_method;
  xt_attrs: attribute_use list;
  xt_wild: ns_wildcard option;
  xt_ct: content_type;
}
and simple_type_definition = {
  st_name: qname option;
  st_variety: variety;
}
and attribute_use = {
  au_required: bool;
  au_decl: attribute_declaration;
  au_cstr: value_constraint;
}
and attribute_group_definition = {
  ag_name: qname;
  ag_attrs: attribute_use list;
  ag_wild: ns_wildcard option;
}
and model_group_definition = {
  mg_name: qname;
  mg_model: model_group
}
and particle = {
  p_min: Big_int.big_int;
  p_max: Big_int.big_int option;
  p_term: term;
}
and wildcard = {
  wc_ns: ns_wildcard;
  wc_process: [ `Skip | `Lax | `Strict ]
}
and schema = {
  sch_types: type_definition list;
  sch_attributes: attribute_declaration list;
  sch_elements: element_declaration list;
  sch_att_groups: attribute_group_definition list;
  sch_model_groups: model_group_definition list
}
and type_definition =
  | Simple of simple_type_definition ptr
  | Complex of complex_type_definition ptr
and term = 
  | Model_group of model_group 
  | Wildcard of wildcard 
  | Element of element_declaration
and model_group = 
  | All of particle list 
  | Choice of particle list 
  | Sequence of particle list
and derivation_method = 
  | Extension 
  | Restriction
and content_type = 
  | Ct_empty 
  | Ct_simple of simple_type_definition 
  | Ct_model of mixed * particle
and mixed = 
  | Mixed 
  | Element_only
and ns_wildcard = 
  | W_any 
  | W_ns of Ns.t list 
  | W_not of Ns.t 
and variety =
  | VRestriction of simple_type_definition ptr * facets
  | VList of simple_type_definition ptr
  | VUnion of simple_type_definition ptr list
