(**************************************************************************)
(*  The CDuce compiler                                                    *)
(*  Alain Frisch <Alain.Frisch@inria.fr> and the CDuce team               *)
(*  Copyright CNRS,INRIA, 2003,2004 (see LICENSE for details)             *)
(**************************************************************************)

open Encodings

type pxp_node =
  ('a Pxp_document.node Pxp_document.extension as 'a) Pxp_document.node
type pxp_document =
  ('a Pxp_document.node Pxp_document.extension as 'a) Pxp_document.document

  (** create a new Pxp configuration. This configuration will be namespace
   * enabled with a brand new namespace manager containing "xsi" and "xsd"
   * schema prefixes declaration *)
val new_xsd_config : unit -> Pxp_types.config

val pxp_document_of :
  ?config: Pxp_types.config -> Pxp_types.source -> pxp_document
val pxp_node_of:
  ?config: Pxp_types.config -> Pxp_types.source -> pxp_node
val pxp_stream_of_file :
  ?config: Pxp_types.config -> string -> Pxp_types.event Stream.t

(* val pxp_stream_of_value : Value.t -> Pxp_yacc.event Stream.t *)

module Pxp_helpers:
  sig
    exception PxpHelpers of exn

    val _tag_name : pxp_node -> Utf8.t

    val _has_attribute : string -> pxp_node -> bool

      (* test if exists a child element which name is given *)
    val _has_element : string -> pxp_node -> bool

      (* test if exists a child element which name is one of the given *)
    val _has_element': string list -> pxp_node -> bool

    val _attribute : string -> pxp_node -> Utf8.t

      (* find the first child element which have the given name *)
    val _element : string -> pxp_node -> pxp_node

      (* find the first child element which name is one of the given names *)
    val _element': string list -> pxp_node -> pxp_node

      (* find all child elements which have a given name *)
    val _elements: string -> pxp_node -> pxp_node list

      (* find all child elements which name is one of the given names *)
    val _elements': string list -> pxp_node -> pxp_node list
  end

  (* namespace handling *)

open Encodings

val has_xsd_prefix : Utf8.t -> bool
val schema_ns_prefixes: (Utf8.t * Utf8.t) list
val xsd_namespace : Ns.t
val xsi_namespace : Ns.t
val xsd_prefix : Utf8.t
val xsi_prefix : Utf8.t

val add_xsd_prefix : Utf8.t -> Utf8.t

  (* misc stuff *)

val string_of_pxp_event : Pxp_types.event -> string

val collect_pcdata : Pxp_types.event Stream.t -> string

  (** @raise Stream.Failure at the end of the stream *)
val peek : Pxp_types.event Stream.t -> Pxp_types.event

  (* debugging only *)
val dump_stream : Pxp_types.event Stream.t -> unit

